// CustomExceptionClass - demonstriert die Flexibilitt des
//                    Ausnahmemechanismus, indem eine benutzerdefinierte
//                    Ausnahmeklasse erstellt wird
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <sstream>
#include <exception>
using namespace std;

class ArgOutOfRange : public exception
{
  protected:
    string sMsg;
    int nValue;
    int nMaxLegal;
    int nMinLegal;

  public:
    ArgOutOfRange(const char* pszFName, int nVal,
                  int nMin = 0, int nMax = 0)
    {
        nValue = nVal;
        nMinLegal = nMin;
        nMaxLegal = nMax;

        ostringstream out;
        out << "Argument auerhalb des Gltigkeitsbereichs in " << pszFName
            << ", arg ist " << nValue;
        if (nMin != nMax)
        {
            out << ", zulssiger Bereich ist "
                << nMin << " to " << nMax;
        }
        out << ends;
        sMsg = out.str();
    }

    virtual const char* what()
    {
        return sMsg.c_str();
    }
};


// factorial - Fakultt berechnen
int factorial(int n)
{
    // Argument muss positiv sein; Ausnahme aufwerfen, wenn
    // n negativ ist
    if (n < 0)
    {
        throw ArgOutOfRange("factorial()", n, 0, 100);
    }

    // alles ber 100 erzeugt einen berlauf
    if (n > 100)
    {
        throw ArgOutOfRange("factorial()", n, 0, 100);
    }

    // jetzt die Fakultt berechnen
    int nAccum = 1;
    while(n > 1)
    {
        nAccum *= n--;
    }
    return nAccum;
}

int main(int nNumberofArgs, char* pszArgs[])
{
    try
    {
        cout << "Fakultt von 3 ist "
             << factorial(3)
             << endl;

        cout << "Fakultt von -1 ist "
             << factorial(-1)
             << endl;

        cout << "Fakultt von 5 ist "
             << factorial(5)
             << endl;
    }
    catch(ArgOutOfRange e)
    {
        cerr << "Fehler aufgetreten:\n" << e.what() << endl;
    }
    catch(...)
    {
        cerr << "Unerwarteter Fehler aufgetreten" << endl;
    }

    // Warten, bis der Benutzer bereit ist, bevor das Programm
    // beendet wird, sodass der Benutzer die Programmergebnisse
    // sehen kann
    cout << "Eingabetaste fuer Weiter..." << endl;

    cin.ignore(10, '\n');
    cin.get();
    return 0;
}
