//
//  PassObjects - dieses Programm demonstriert die bergabe eines
//                Objekts als Wert im Vergleich zur bergabe der
//                Adresse des Objekts
//
#include <cstdio>
#include <cstdlib>
#include <iostream>
#include <cstring>
using namespace std;


// Savings - eine einfache Sparkonto-Klasse
class Savings
{
  public:
    int    nAccountNumber;
    double dBalance;

    // deposit - Einzahlung eines Betrags auf das Konto;
    //           Einzahlungen mssen positiv sein; Rckgabe
    //           ist der resultierende Kontostand oder
    //           0 im Fehlerfall
    double deposit(double dAmount)
    {
        // keine negativen Einzahlungen - das ist eine Abhebung
        if (dAmount < 0)
        {
            return 0.0;
        }

        // okay - add to the balance and return the total
        dBalance += dAmount;
        return dBalance;
    }

    // withdraw - ein Abhebung ausfhren, wenn gengend Geld
    //            verfgbar ist
    double withdraw(double dAmount)
    {
        if (dBalance < dAmount)
        {
            return 0.0;
        }

        dBalance -= dAmount;
        return dBalance;
    }

    // balance - Rckgabe des Kontostands des aktuellen Objekts
    double balance()
    {
        return dBalance;
    }
};

// someFunction(Savings) - Objekt als Wert entgegennehmen
void someFunction(Savings s)
{
    cout << "In someFunction(Savings)" << endl;

    cout << "Depositing $100" << endl;
    s.deposit(100.0);

    cout << "Balance in someFunction(Savings) is "
         << s.balance() << endl;
}

// someFunction(Savings*) - Adresse des Objekts entgegennehmen
void someFunction(Savings* pS)
{
    cout << "In someFunction(Savings*)" << endl;

    cout << "Einzahlung $100" << endl;
    pS->deposit(100.0);

    cout << "Kontostand in someFunction(Savings) ist "
         << pS->balance() << endl;
}

int main(int nNumberofArgs, char* pszArgs[])
{
    Savings s = {0, 0.0};

    // zuerst bergabe als Wert
    someFunction(s);
    cout << "Kontostand zurck in main() ist "
         << s.balance() << endl;

    // jetzt die Adresse bergeben
    someFunction(&s);
    cout << "Kontostand zurck in main() ist "
         << s.balance() << endl;

    // Warten, bis der Benutzer bereit ist, bevor das Programm
    // beendet wird, sodass der Benutzer die Programmergebnisse
    // sehen kann
    cout << "Eingabetaste fuer Weiter..." << endl;

    cin.ignore(10, '\n');
    cin.get();
    return 0;
}
